@@ -36,11 +36,13 @@
 
 #include <stdio.h>
 #include <unistd.h>
+#include <fcntl.h>
 #include <getopt.h>
 #include <string.h>
 #include <netdb.h>
 #include <errno.h>
 #include <sys/param.h>
+#include <sys/stat.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #ifdef STANDALONE
@@ -55,7 +57,7 @@
 #include "net-locale.h"
 
 char *Release = RELEASE,
-     *Version = "hostname 2.01";
+     *Version = "hostname 2.05";
      
 static char *program_name;
 static int  opt_v;
@@ -67,6 +69,9 @@ static void usage(void);
 static void version(void);
 static void setfilename(char *,int);
 
+char * localhost();
+char * localdomain();
+
 #define SETHOST		1
 #define SETDOMAIN	2
 
@@ -119,7 +124,7 @@ static void showhname(char *hname, int c)
 	struct hostent *hp;
 	register char *p,**alias;
 	struct in_addr **ip;
-	
+	int flag = 0;	
 	if (opt_v)
 		fprintf(stderr,NLS_CATGETS(catfd, hostnameSet, hostname_verb_res, "Resolving `%s' ...\n"),hname);
 	if (!(hp = gethostbyname(hname))) {
@@ -147,13 +152,21 @@ static void showhname(char *hname, int c)
 
 	switch(c) {
 		case 'a':
-			while (hp->h_aliases[0])
-				printf("%s ", *hp->h_aliases++);
+			while (hp->h_aliases[0]) {
+				if(flag)
+					printf(" ");
+				printf("%s", *hp->h_aliases++);
+				flag = 1;
+			}
 			printf("\n");
 			break;
 		case 'i':
-			while (hp->h_addr_list[0])
+			while (hp->h_addr_list[0]) {
+				if(flag)
+					printf(" ");
 				printf("%s ", inet_ntoa(*(struct in_addr *) * hp->h_addr_list++));
+				flag = 1;
+			}
 			printf("\n");
 			break;
 		case 'd':
@@ -172,26 +185,40 @@ static void showhname(char *hname, int c)
 
 static void setfilename(char * name,int what)
 {		
-  register FILE *fd;
-  register char *p;
-  char fline[MAXHOSTNAMELEN];
+  register int fd;
+  register char *p, *q;
+  char *buf;
+  struct stat fs;
 
-  if ((fd = fopen(name, "r")) != NULL) {
-	while (fgets(fline, sizeof(fline), fd) != NULL) {
-		if ((p = strchr(fline, '\n')) != NULL)
-			*p = '\0';
-			if (opt_v) fprintf(stderr, ">> %s\n", fline);
-			if (fline[0] == '#')
-				continue;
-			if ((p = strrchr(fline, ' ')) != NULL)
-				*p = '\0';
-			if (what == SETHOST) {
-				sethname(fline);
-			} else {
-				setdname(fline);
-			}
+  if ((fd = open(name, O_RDONLY)) != -1 && fstat(fd, &fs) != -1) {
+       if (NULL == (buf = calloc(fs.st_size + 1, sizeof(char)))) {
+               fprintf(stderr, "%s: memory exhaustion\n", program_name);
+               NLS_CATCLOSE(catfd)
+               exit(1);
 	}
-	(void) fclose(fd);
+       if (fs.st_size != read(fd, buf, fs.st_size)) {
+               fprintf(stderr, "%s: can't read `%s'\n", program_name, optarg);
+               NLS_CATCLOSE(catfd)
+               exit(1);
+       }
+
+       buf[fs.st_size] = '\0';
+       p = buf;
+       while (*p == '#')
+               if ((q = strchr(p, '\n')) != NULL)
+                       p = q + 1;
+               else
+                       p = buf + fs.st_size;
+       if ((q = strchr(p, '\n')) != NULL)
+                       *q = '\0';
+       if (opt_v) fprintf(stderr, ">> %s\n", p);
+       if ((q = strrchr(p, ' ')) != NULL)
+               *q = '\0';
+       if (what == SETHOST)
+               sethname(p);
+       else
+               setdname(p);
+       (void) close(fd);
   } else {
 	fprintf(stderr,NLS_CATGETS(catfd, hostnameSet, hostname_cant_open, "%s: can't open `%s'\n"),
 			program_name, optarg);
@@ -259,7 +286,7 @@ int main(int argc, char **argv)
 	char type='\0';
 	int option_index = 0;
 	int what = 0;
-	char myname[MAXHOSTNAMELEN+1];
+       char *myname;
 	char *file=NULL;
 
 	static const struct option long_options[] =
@@ -353,7 +380,7 @@ int main(int argc, char **argv)
 				break;
 			}
 		case 1:
-			gethostname(myname, sizeof(myname));
+                       myname = localhost();
 			if (opt_v)
 				fprintf(stderr,NLS_CATGETS(catfd, hostnameSet, hostname_verb_get, 
 					"gethostname()=`%s'\n"),myname);
@@ -371,7 +398,7 @@ int main(int argc, char **argv)
 				setdname(argv[optind]);
 				break;
 			}
-			getdomainname(myname,sizeof(myname));
+                       myname = localdomain();
 			if (opt_v)
 				fprintf(stderr,NLS_CATGETS(catfd, hostnameSet, hostname_verb_dget, 
 					"getdomainname()=`%s'\n"),myname);
